<?php

declare(strict_types=1);

namespace Drupal\Tests\devel_wizard\Kernel\Plugin\DevelWizard\Spell;

use Drupal\Tests\devel_wizard\Traits\TemplateTrait;

/**
 * @code
 * ../vendor/bin/phpunit \
 * --configuration='../managedDrupalExtension/devel_wizard/phpunit.xml' \
 * --filter='testTemplateConfigEntity with data set "spell/entity_type/config/list_builder.php.twig/.*"' \
 * modules/contrib/devel_wizard/tests/src/Kernel/Plugin/DevelWizard/Spell/EntityTypeSpellTest.php
 * @endcode
 */
class EntityTypeSpellTest extends SpellTestBase {

  use TemplateTrait;

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'devel_wizard',
  ];

  public function casesTemplateConfigEntityClass(): array {
    return $this->collectTemplateCases('spell/entity_type/config/class.php.twig');
  }

  public function casesTemplateConfigEntityInterface(): array {
    return $this->collectTemplateCases('spell/entity_type/config/interface.php.twig');
  }

  public function casesTemplateConfigEntityAddForm(): array {
    return $this->collectTemplateCases('spell/entity_type/config/add_form.php.twig');
  }

  public function casesTemplateConfigEntityEditForm(): array {
    return $this->collectTemplateCases('spell/entity_type/config/edit_form.php.twig');
  }

  public function casesTemplateConfigEntityDeleteForm(): array {
    return $this->collectTemplateCases('spell/entity_type/config/delete_form.php.twig');
  }

  public function casesTemplateConfigEntityAccessControlHandler(): array {
    return $this->collectTemplateCases('spell/entity_type/config/access_control_handler.php.twig');
  }

  public function casesTemplateConfigEntityListBuilder(): array {
    return $this->collectTemplateCases('spell/entity_type/config/list_builder.php.twig');
  }

  public function casesTemplateConfigEntityRouteProvider(): array {
    return $this->collectTemplateCases('spell/entity_type/config/route_provider.php.twig');
  }

  public function casesTemplateConfigEntityStorage(): array {
    return $this->collectTemplateCases('spell/entity_type/config/storage.php.twig');
  }

  public function casesTemplateConfigEntityComparer(): array {
    return $this->collectTemplateCases('spell/entity_type/config/comparer.php.twig');
  }

  public function casesTemplateConfigEntityPermissionProviderInterface(): array {
    return $this->collectTemplateCases('spell/entity_type/config/permission_provider_interface.php.twig');
  }

  public function casesTemplateConfigEntityPermissionProvider(): array {
    return $this->collectTemplateCases('spell/entity_type/config/permission_provider.php.twig');
  }

  /**
   * @dataProvider casesTemplateConfigEntityClass
   * @dataProvider casesTemplateConfigEntityInterface
   * @dataProvider casesTemplateConfigEntityAddForm
   * @dataProvider casesTemplateConfigEntityEditForm
   * @dataProvider casesTemplateConfigEntityDeleteForm
   * @dataProvider casesTemplateConfigEntityAccessControlHandler
   * @dataProvider casesTemplateConfigEntityListBuilder
   * @dataProvider casesTemplateConfigEntityRouteProvider
   * @dataProvider casesTemplateConfigEntityStorage
   * @dataProvider casesTemplateConfigEntityComparer
   * @dataProvider casesTemplateConfigEntityPermissionProviderInterface
   * @dataProvider casesTemplateConfigEntityPermissionProvider
   */
  public function testTemplateConfigEntity(string $expected, string $template, array $context): void {
    $container = \Drupal::getContainer();
    $twig = $container->get('twig');
    /** @noinspection PhpUnhandledExceptionInspection */
    static::assertSame(
      $expected,
      $twig->render($template, $context),
    );
  }

}
